import os

from django.core.exceptions import ValidationError
from django.forms.utils import ErrorList
from django.utils.functional import cached_property
from django.utils.safestring import mark_safe
from django.utils.text import slugify

import magic
from wagtail.core import blocks
from wagtail.core.blocks.field_block import ChoiceBlock
from wagtail.documents.blocks import DocumentChooserBlock
from wagtail.embeds.blocks import EmbedBlock
from wagtail.images.blocks import ImageChooserBlock
from wagtailmedia.blocks import (
    AudioChooserBlock as WagtailMediaAudioChooserBlock,
)

from lens.blog.blocks import LastArticlesBlock
from lens.demarches.blocks import DemarchesBlock
from lens.events.blocks import UpcomingEventsBlock
from lens.utils.blocks import (
    HideableStructBlock,
    Style,
    StylesChoiceBlock,
    StylizedBlockMixin,
)

"""Liste des icônes disponibles."""
ICON_CHOICES = (
    ('phone', "Téléphone"),
    ('location', "Emplacement"),
    ('clock', "Horloge"),
    ('calendar', "Calendrier"),
    ('family', "Famille"),
    ('globe', "Globe"),
    ('mail', "Enveloppe"),
    ('ticket', "Billet"),
)

"""Liste des styles d'un bouton."""
BUTTON_STYLES = (
    Style('primary', "Principal (rouge)", 'btn-primary'),
    Style('secondary', "Secondaire (bleu)", 'btn-secondary'),
    Style('outline-primary', "Principal sans fond", 'btn-outline-primary'),
    Style(
        'outline-secondary', "Secondaire sans fond", 'btn-outline-secondary'
    ),
    Style(
        'outline-primary-lg',
        "Principal sans fond, large",
        'btn-outline-primary btn-lg',
    ),
    Style(
        'outline-secondary-lg',
        "Secondaire sans fond, large",
        'btn-outline-secondary btn-lg',
    ),
    Style(
        'outline-dark-lg', "Sombre sans fond, large", 'btn-outline-dark btn-lg'
    ),
)

"""Liste des styles d'une fiche."""
CARD_STYLES = (
    Style('default', "Défaut", ''),
    Style('primary', "Coloré : principal (rouge)", 'text-white bg-primary'),
    Style(
        'secondary', "Coloré : secondaire (bleu)", 'text-white bg-secondary'
    ),
    Style('dark', "Coloré : sombre (gris)", 'text-white bg-dark'),
    Style(
        'primary-sm',
        "Petit et coloré : principal (rouge)",
        'card-sm text-white bg-primary',
    ),
)


class BlockGroup:
    LAYOUT = "Agencement"
    IMAGES = "Images"
    MEDIA = "Média"


# TYPOGRAPHIE
# ------------------------------------------------------------------------------


class TitleStructValue(blocks.StructValue):
    """
    Valeur du bloc d'un titre.
    """

    @cached_property
    def anchor(self):
        """Génère une ancre basée sur le titre."""
        return slugify(self.get('title'))


class TitleBlock(HideableStructBlock):
    """
    Bloc d'un titre avec choix de son niveau.
    """

    title = blocks.CharBlock(label="Titre", classname='title')
    level = blocks.ChoiceBlock(
        choices=[
            ('h{}'.format(i), "Titre de niveau {}".format(i))
            for i in range(3, 7)
        ],
        default='h3',
        label="Niveau",
        help_text=(
            "Choix du niveau de titre. Les niveaux de titre permettant "
            "de définir la structure du document."
        ),
    )

    class Meta:
        label = "Titre"
        icon = 'title'
        template = 'blocks/title_block.html'
        value_class = TitleStructValue
        closed = True


class ParagraphBlock(blocks.RichTextBlock):
    """
    Bloc d'un paragraphe, proposant un éditeur minimal.
    """

    features = (
        'bold',
        'italic',
        'ol',
        'ul',
        'hr',
        'link',
        'document-link',
        'color-primary',
        'color-secondary',
    )

    def __init__(self, features=None, **kwargs):
        if features is None:
            features = self.features
        return super().__init__(features=features, **kwargs)

    class Meta:
        label = "Paragraphe"
        icon = 'pilcrow'
        template = 'blocks/paragraph_block.html'
        closed = True


class ParagraphStructBlock(HideableStructBlock):
    """
    Bloc d'un paragraphe, pouvant être visible par le public ou non
    """

    features = (
        'bold',
        'italic',
        'ol',
        'ul',
        'hr',
        'link',
        'document-link',
        'color-primary',
        'color-secondary',
    )

    text = blocks.RichTextBlock(
        features=features,
        label="Paragraphe",
    )

    class Meta:
        label = "Paragraphe"
        icon = 'pilcrow'
        template = 'blocks/paragraph_struct_block.html'
        closed = True


class QuoteBlock(HideableStructBlock):
    """
    Bloc d'une citation, d'un auteur optionnel.
    """

    text = ParagraphBlock(label="Texte")
    author = blocks.CharBlock(label="Auteur", required=False)

    class Meta:
        label = "Citation"
        icon = 'openquote'
        template = 'blocks/quote_block.html'
        closed = True


# LIENS ET BOUTONS
# ------------------------------------------------------------------------------


class LinkStructValue(blocks.StructValue):
    """
    Valeur du bloc d'un lien.
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.url = self.get_url()

    def get_url(self):
        """Détermine et retourne l'URL du lien."""
        url = ''
        for name in self.block.LINK_BLOCKS:
            value = self.get(name)
            if not value:
                continue
            if name == 'page':
                url = value.full_url
            elif name == 'document':
                url = value.url
            elif name == 'external':
                url = value
            # ajoute l'ancre à l'URL
            if name == 'anchor' or self.get('anchor'):
                url = '{}#{}'.format(url, self.get('anchor').lstrip('#'))
            return url


class LinkBlock(blocks.StructBlock):
    """
    Bloc d'un lien pointant vers une ressource interne ou externe.
    """

    page = blocks.PageChooserBlock(label="Page", required=False)
    document = DocumentChooserBlock(label="Document", required=False)
    external = blocks.URLBlock(label="URL", required=False)
    anchor = blocks.CharBlock(
        label="Ancre",
        required=False,
        help_text=mark_safe(
            "Une ancre (ex. : <code>#id-cible</code>) à ajouter à la page "
            "sélectionnée ou à celle visitée."
        ),
    )

    LINK_BLOCKS = ('page', 'document', 'external', 'anchor')

    class Meta:
        label = "Lien"
        icon = 'link'
        required = True
        value_class = LinkStructValue
        help_text = (
            "Un lien vers lequel pointer au clic. Une seule cible est "
            "possible parmi les suivantes."
        )
        closed = True

    def clean(self, value):
        """Vérifie les champs et la cible définie."""
        value = super().clean(value)

        link_keys = {k for k in self.LINK_BLOCKS if value.get(k)}

        if not link_keys and self.required:
            raise ValidationError(
                "Erreur de validation pour LinkBlock",
                params=dict.fromkeys(
                    self.LINK_BLOCKS,
                    ErrorList(["Une cible est requise pour ce lien."]),
                ),
            )
        if 'anchor' in link_keys and len(link_keys) == 2:
            if 'page' not in link_keys:
                raise ValidationError(
                    "Erreur de validation pour LinkBlock",
                    params={
                        'anchor': ErrorList(
                            [
                                "L'ancre n'est que valable seule ou avec une "
                                "page."
                            ]
                        )
                    },
                )
        elif len(link_keys) > 1:
            raise ValidationError(
                "Erreur de validation pour LinkBlock",
                params=dict.fromkeys(
                    link_keys,
                    ErrorList(["Le lien ne peut pointer que vers une cible."]),
                ),
            )

        return value

    @property
    def required(self):
        return self.meta.required


class ButtonBlock(HideableStructBlock):
    """
    Bloc d'un bouton pointant vers un lien.
    """

    text = blocks.CharBlock(label="Texte", max_length=100)
    link = LinkBlock(required=True)
    size = ChoiceBlock(
        choices=[
            ('btn-small', "petit"),
            ('btn-medium', "moyen"),
            ('btn-large', "gros"),
        ],
        blank=True,
        required=False,
        label="Taille",
        help_text="Choix de la taille du bouton, laisser vide si indifférent.",
    )

    class Meta:
        label = "Bouton"
        icon = 'tick-inverse'
        template = 'blocks/button_block.html'
        closed = True

        """La classe CSS à ajouter au bouton."""
        button_css_class = None

    def get_css_class(self, value):
        """Retourne la classe CSS à appliquer au bouton."""
        return self.meta.button_css_class or None

    def get_context(self, value, **kwargs):
        """
        Ajoute la classe CSS du bouton au contexte,
        ainsi que l'attribut target_blank
        """
        context = super().get_context(value, **kwargs)
        context.setdefault('css_class', self.get_css_class(value))
        context['target_blank'] = False
        if value['link']['document'] or value['link']['external']:
            context['target_blank'] = True
        return context


class PDFBlock(HideableStructBlock):
    """
    Bloc d'une liseuse PDF
    """

    file = DocumentChooserBlock(label="Document PDF")

    class Meta:
        label = "Liseuse PDF"
        icon = "doc-full"
        template = "blocks/pdf_block.html"
        group = BlockGroup.MEDIA

    def clean(self, value):
        validation_error = ValidationError(
            'Veuillez sélectionner un fichier au format PDF.'
        )
        file_ = value.get('file').file
        file_mime_type = magic.from_buffer(file_.read(1024), mime=True)
        if file_mime_type != 'application/pdf':
            raise validation_error

        _, ext = os.path.splitext(file_.name)
        if ext.lower() != '.pdf':
            raise validation_error

        return super().clean(value)


class StylizedButtonBlock(StylizedBlockMixin, ButtonBlock):
    """
    Bloc d'un bouton avec différents styles possibles.
    """

    style = StylesChoiceBlock(
        BUTTON_STYLES,
        label="Style",
        help_text="Définis l'apparence qu'aura le bouton.",
    )

    def get_css_class(self, value):
        # Récupère la classe depuis le style définis
        if hasattr(value, 'css_class'):
            return value.css_class
        return super().get_css_class(value)


# IMAGES
# ------------------------------------------------------------------------------


class ImageBlock(HideableStructBlock):
    """
    Bloc d'une image avec une légende et un lien optionnels.
    """

    image = ImageChooserBlock(label="Image")
    caption = blocks.CharBlock(
        required=False,
        label="Légende",
        help_text=(
            "Une description de l'image, qui sera affichée en dessous."
        ),
    )
    expandable = blocks.BooleanBlock(
        label="Image agrandissable au clic",
        default=False,
        required=False,
        help_text=(
            "Si le bloc contient un lien, ce paramètre"
            "n'influera pas sur son comportement."
        ),
    )
    link = LinkBlock(required=False)

    class Meta:
        label = "Image"
        icon = 'image'
        template = 'blocks/image_block.html'
        group = BlockGroup.IMAGES
        closed = True

    def get_context(self, value, **kwargs):
        """Ajoute l'attribut target_blank du bouton au contexte"""
        context = super().get_context(value, **kwargs)
        context['target_blank'] = False
        if value['link']['document'] or value['link']['external']:
            context['target_blank'] = True
        return context


class ImageTextBlock(ImageBlock):
    """
    Bloc d'une image avec du texte par-dessus.
    """

    text = ParagraphBlock(
        label="Texte", help_text="Texte qui apparaitra par-dessus l'image."
    )
    is_animated = blocks.BooleanBlock(
        required=False,
        label="Animation au survol",
        help_text=(
            "L'image sera assombrie, sauf au survol, où le texte "
            "disparaîtra avec une transition."
        ),
    )

    class Meta:
        label = "Image avec texte"
        icon = 'image'
        template = 'blocks/image_text_block.html'
        group = BlockGroup.IMAGES
        closed = True


class ImageCarouselBlock(HideableStructBlock):
    """
    Bloc d'un carrousel d'images, avec des liens optionnels.
    """

    items = blocks.StreamBlock(
        [('image', ImageBlock())], min_num=1, label="Éléments"
    )

    class Meta:
        label = "Carrousel d'images"
        icon = 'image'
        template = 'blocks/image_carousel_block.html'


class IconTextBlock(StylizedBlockMixin, HideableStructBlock):
    """
    Bloc d'une icône avec du texte en dessous.
    """

    icon = blocks.ChoiceBlock(choices=ICON_CHOICES, label="Icône")
    text = ParagraphBlock(
        label="Texte", help_text="Texte qui apparaitra en dessous de l'icône."
    )

    class Meta:
        label = "Icône avec texte"
        icon = 'radio-empty'
        template = 'blocks/icon_text_block.html'
        closed = True


class AudioChooserBlock(WagtailMediaAudioChooserBlock):
    class Meta:
        template = 'blocks/audio_block.html'
        label = "Audio"
        icon = 'media'
        group = BlockGroup.MEDIA


# CONTENEURS
# ------------------------------------------------------------------------------


class MediaObjectBlock(HideableStructBlock):
    """
    Bloc d'un object multimédia, avec une image et du texte enrichi.
    """

    image = ImageChooserBlock(label="Image")
    section_blocks = blocks.StreamBlock(
        [
            (
                'paragraph_block',
                ParagraphBlock(
                    features=(
                        'h5',
                        'h6',
                        'bold',
                        'italic',
                        'ol',
                        'ul',
                        'link',
                        'document-link',
                    )
                ),
            ),
            ('button_block', StylizedButtonBlock()),
        ],
        min_num=1,
        label="Contenu",
    )

    class Meta:
        label = "Objet multimédia"
        icon = 'image'
        template = 'blocks/media_object_block.html'
        group = BlockGroup.LAYOUT
        closed = True


class CardBlock(StylizedBlockMixin, HideableStructBlock):
    """
    Bloc basé sur le composant Card, pouvant contenir, en plus du contenu,
    une image, un titre et un sous-titre optionnels.
    """

    top_image = ImageChooserBlock(label="Image d'en-tête", required=False)
    title = blocks.CharBlock(label="Titre", required=False)
    subtitle = blocks.CharBlock(label="Sous-titre", required=False)
    body_blocks = blocks.StreamBlock(
        [
            ('paragraph_block', ParagraphBlock()),
            ('button_block', ButtonBlock(button_css_class='btn-card')),
        ],
        min_num=1,
        label="Contenu",
    )
    style = StylesChoiceBlock(
        CARD_STYLES,
        required=False,
        label="Style",
        help_text="Définis l'apparence qu'aura la fiche.",
    )

    class Meta:
        label = "Fiche"
        icon = 'form'
        template = 'blocks/card_block.html'
        group = BlockGroup.LAYOUT
        closed = True


class BaseStreamBlock(blocks.StreamBlock):
    """
    Ensemble des blocs de contenu de base.
    """

    title_block = TitleBlock()
    paragraph_structblock = ParagraphStructBlock()
    button_block = StylizedButtonBlock()
    icon_text_block = IconTextBlock()
    quote_block = QuoteBlock()

    image_block = ImageBlock()
    image_text_block = ImageTextBlock()

    embed_block = EmbedBlock(
        label="Média intégré",
        max_width=1280,
        max_height=720,
        group=BlockGroup.MEDIA,
    )
    audio_block = AudioChooserBlock()

    card_block = CardBlock()
    media_object_block = MediaObjectBlock()
    pdf_block = PDFBlock()


class Grid:
    """
    Définis les choix et valeurs ainsi que des méthodes pour les blocs
    liés à une grille.
    """

    #: Valeur pour une même largeur des colonnes.
    COLUMNS_EQUAL = '0'
    #: Valeur pour la largeur naturelle d'une colonne.
    COLUMNS_AUTO = 'auto'

    #: Nombre maximal de colonnes sur une ligne.
    NB_COLUMNS_MAX = 12

    #: Liste des choix pour la largeur des colonnes.
    NB_COLUMNS_CHOICES = [
        (COLUMNS_EQUAL, "largeurs égales"),
        (COLUMNS_AUTO, "largeur du contenu"),
    ] + [(nb, str(nb)) for nb in (1, 2, 3, 4, 6, 12)]

    #: Valeur par défaut pour la taille d'une colonne.
    DEFAULT_COLUMN_SIZE = COLUMNS_EQUAL

    #: Liste des choix d'alignement horizontal.
    HORIZONTAL_ALIGNMENT_CHOICES = (
        ('left', "Gauche"),
        ('center', "Centré"),
        ('right', "Droite"),
    )

    @classmethod
    def get_column_size_css_class(cls, size, breakpoint=None):
        """
        Retourne la classe CSS à appliquer à une colonne pour une
        résolution et une taille de colonne données.
        """
        css_class = 'col' if not breakpoint else 'col-{}'.format(breakpoint)
        if size == cls.COLUMNS_EQUAL:
            return css_class
        if size == cls.COLUMNS_AUTO:
            return '{}-auto'.format(css_class)
        try:
            return '{}-{}'.format(
                css_class, int(cls.NB_COLUMNS_MAX / int(size))
            )
        except ValueError:
            return ''

    @classmethod
    def get_column_css_class(cls, sizes, halign=None):
        """
        Retourne la ou les classes CSS à appliquer à une colonne pour
        différentes tailles données par `sizes`, une liste de 2-tuple
        (breakpoint, size).
        """
        css_classes = [
            cls.get_column_size_css_class(size, breakpoint)
            for breakpoint, size in sizes or ((None, cls.DEFAULT_COLUMN_SIZE),)
        ]
        if halign:
            if halign == 'left':
                css_classes.append('text-left')
            elif halign == 'center':
                css_classes.append('text-center')
            elif halign == 'right':
                css_classes.append('text-right')
        return ' '.join(css_classes)

    @staticmethod
    def get_row_css_class(halign):
        """Retourne la classe CSS d'alignement vertical d'une ligne."""
        if halign == 'left':
            return 'justify-content-start'
        if halign == 'center':
            return 'justify-content-center'
        if halign == 'right':
            return 'justify-content-end'
        return ''


class ColumnBlock(HideableStructBlock):
    """
    Bloc d'une colonne contenant un bloc de contenu, prévu pour être affiché
    dans une ligne.
    """

    size_sm = blocks.ChoiceBlock(
        required=False,
        choices=Grid.NB_COLUMNS_CHOICES,
        label="Taille de la colonne (petit)",
    )
    size_md = blocks.ChoiceBlock(
        required=False,
        choices=Grid.NB_COLUMNS_CHOICES,
        label="Taille de la colonne (moyen)",
    )
    size_lg = blocks.ChoiceBlock(
        required=False,
        choices=Grid.NB_COLUMNS_CHOICES,
        label="Taille de la colonne (large)",
    )
    horizontal_align = blocks.ChoiceBlock(
        required=False,
        choices=Grid.HORIZONTAL_ALIGNMENT_CHOICES,
        label="Alignement horizontal",
    )
    block = BaseStreamBlock(label="Contenu", min_num=1)

    class Meta:
        label = "Colonne"
        template = 'blocks/column_block.html'
        group = BlockGroup.LAYOUT
        closed = True

    @cached_property
    def definition(self):
        definition = super().definition
        # évite d'avoir le nombre comme titre du bloc
        definition.pop('titleTemplate', None)
        return definition

    def get_context(self, value, **kwargs):
        context = super().get_context(value, **kwargs)

        # ajoute la classe de la colonne pour les différentes résolutions
        context['column_css_class'] = Grid.get_column_css_class(
            [
                (breakpoint, value['size_{}'.format(breakpoint)])
                for breakpoint in ('sm', 'md', 'lg')
                if value['size_{}'.format(breakpoint)]
            ],
            value['horizontal_align'],
        )

        # permets l'accès au contenu via 'block' afin de permettre une
        # compatibilité avec l'inclusion depuis blocks/row_block.html
        context['block'] = value['block']

        return context


class RowBlock(HideableStructBlock):
    """
    Bloc contenant un ou plusieurs blocs de contenus disposés en colonnes sur
    une ligne.
    """

    nb_columns_sm = blocks.ChoiceBlock(
        required=False,
        choices=Grid.NB_COLUMNS_CHOICES,
        default=Grid.COLUMNS_EQUAL,
        label="Nombre de colonnes (petit)",
        help_text=(
            "La taille des colonnes au sein de cette ligne pour cette "
            "résolution. Vous pouvez choisir entre un nombre fixe de "
            "colonnes, utiliser la même largeur pour toutes les colonnes "
            "ou adapter la largeur au contenu de la colonne."
        ),
    )
    nb_columns_md = blocks.ChoiceBlock(
        required=False,
        choices=Grid.NB_COLUMNS_CHOICES,
        label="Nombre de colonnes (moyen)",
    )
    nb_columns_lg = blocks.ChoiceBlock(
        required=False,
        choices=Grid.NB_COLUMNS_CHOICES,
        label="Nombre de colonnes (large)",
    )
    horizontal_align = blocks.ChoiceBlock(
        required=False,
        choices=Grid.HORIZONTAL_ALIGNMENT_CHOICES,
        label="Alignement horizontal",
    )
    blocks = BaseStreamBlock(
        [('column_block', ColumnBlock())], label="Colonnes", min_num=1
    )

    class Meta:
        label = "Ligne"
        icon = 'placeholder'
        template = 'blocks/row_block.html'
        group = BlockGroup.LAYOUT
        closed = True

    @cached_property
    def definition(self):
        definition = super().definition
        # évite d'avoir le nombre comme titre du bloc
        definition.pop('titleTemplate', None)
        return definition

    def get_context(self, value, **kwargs):
        context = super().get_context(value, **kwargs)

        # ajoute la classe des colonnes pour les différentes résolutions
        context['column_css_class'] = Grid.get_column_css_class(
            [
                (breakpoint, value['nb_columns_{}'.format(breakpoint)])
                for breakpoint in ('sm', 'md', 'lg')
                if value['nb_columns_{}'.format(breakpoint)]
            ],
            value['horizontal_align'],
        )
        context['default_column_css_class'] = 'col-sm'

        # ajoute la classe de la ligne pour l'alignement
        context['row_css_class'] = Grid.get_row_css_class(
            value['horizontal_align']
        )

        return context


class NestedRowBlock(RowBlock):
    """
    Bloc contenant un ou plusieurs blocs de contenus, ou d'une ligne,
    disposés en colonnes sur une ligne.
    """

    blocks = BaseStreamBlock(
        [('column_block', ColumnBlock()), ('row_block', RowBlock())],
        label="Colonnes",
        min_num=1,
    )


class SectionBlock(HideableStructBlock):
    """
    Bloc d'une section, composée d'un autre bloc et d'un titre optionnel.
    """

    title = blocks.CharBlock(label="Titre", required=False)
    content = blocks.StreamBlock(
        [
            ('image_carousel', ImageCarouselBlock(closed=False)),
            ('last_articles', LastArticlesBlock(closed=False)),
            ('upcoming_events', UpcomingEventsBlock(closed=False)),
            ('demarches', DemarchesBlock(closed=False)),
        ],
        min_num=1,
        max_num=1,
        label="Contenu",
    )

    class Meta:
        label = "Section"
        icon = 'folder'
        template = 'blocks/section_block.html'
        closed = True


# ENSEMBLES DE BLOCS
# ------------------------------------------------------------------------------


class BodyStreamBlock(BaseStreamBlock):
    """
    Ensemble des blocs disponibles pour le contenu une page.
    """

    row_block = NestedRowBlock()


class HomeStreamBlock(blocks.StreamBlock):
    """
    Ensemble des blocs disponibles pour la page d'accueil.
    """

    section_block = SectionBlock()
