import logging
from collections import OrderedDict

from django.conf import settings
from django.core.exceptions import ValidationError
from django.db import models
from django.forms.utils import flatatt
from django.utils.functional import cached_property
from django.utils.safestring import mark_safe

import magic
from wagtail.admin.edit_handlers import (
    FieldPanel,
    MultiFieldPanel,
    ObjectList,
    PageChooserPanel,
    StreamFieldPanel,
    TabbedInterface,
)
from wagtail.contrib.settings.models import BaseSetting, register_setting
from wagtail.core.blocks import StreamBlock
from wagtail.core.fields import RichTextField, StreamField
from wagtail.core.models import Page
from wagtail.images.edit_handlers import ImageChooserPanel
from wagtail.search import index
from wagtailmedia.models import AbstractMedia
from wagtailmenus.models import AbstractLinkPage

from .blocks import BodyStreamBlock, ButtonBlock, HomeStreamBlock

logger = logging.getLogger(__name__)


class SourceMedia(AbstractMedia):
    """
    Source de différents types de média (cf. `AbstractMedia.MEDIA_TYPES`).
    """

    duration = models.PositiveIntegerField("durée", blank=True, null=True)
    mimetype = models.CharField(
        max_length=20, blank=True, null=True, editable=False
    )

    admin_form_fields = ('title', 'file', 'collection', 'tags')

    def get_source_attrs(self):
        """Retourne un dictionnaire des attributs de la balise <source>."""
        attrs = OrderedDict()
        if self.file:
            attrs['src'] = self.url
        if self.mimetype:
            attrs['type'] = self.mimetype
        return attrs

    def source_tag(self, extra_attributes={}):
        """Retourne le code HTML de la balise <source> de ce média."""
        attrs = self.get_source_attrs()
        attrs.update(extra_attributes)
        return mark_safe('<source{}>'.format(flatatt(attrs)))

    def clean(self):
        # valide le type de fichier
        if self.type and self.file:
            try:
                mimetype = magic.from_buffer(self.file.read(1024), mime=True)
            except Exception:
                logger.exception(
                    "impossible de déterminer le type mime du fichier %s",
                    self.file,
                )
                raise ValidationError(
                    {
                        'file': ValidationError(
                            "Impossible de déterminer le type de fichier.",
                            code='invalid',
                        )
                    }
                )
            else:
                if not mimetype.startswith('{}/'.format(self.type)):
                    raise ValidationError(
                        {
                            'file': ValidationError(
                                "Le fichier ne semble pas de type {}.".format(
                                    self.type
                                ),
                                code='invalid_mimetype',
                            )
                        }
                    )
                self.mimetype = mimetype


@register_setting(icon='help')
class OrganizationInfo(BaseSetting):
    """
    Informations sur l'organisation utiles pour le site, comprenant :
     * les informations pratiques et liens externes ;
     * les informations légales et la configuration du suivi.
    """

    # Informations pratiques
    phone = models.CharField("téléphone", max_length=20, blank=True)
    contact_link = models.URLField(
        "lien de contact",
        blank=True,
        help_text=(
            "L'URL d'une page permettant de vous contacter, via un "
            "formulaire par exemple."
        ),
    )
    address = models.CharField(
        "adresse",
        max_length=255,
        blank=True,
        help_text="L'adresse postale affichée sur une ligne.",
    )
    opening_hours = models.CharField(
        "horaires d'ouverture", max_length=255, blank=True
    )
    map_link = models.URLField(
        "lien vers une carte",
        blank=True,
        help_text=mark_safe(
            "L'URL d'une carte afin de situer la ville, par exemple "
            "sur <a href=\"https://openstreetmap.org\">OpenStreetMap</a>."
        ),
    )

    # Liens vers les réseaux
    facebook_link = models.URLField("Facebook", blank=True)
    instagram_link = models.URLField("Instagram", blank=True)
    youtube_link = models.URLField("YouTube", blank=True)

    # Informations légales
    legal_notice_page = models.ForeignKey(
        'wagtailcore.Page',
        null=True,
        on_delete=models.SET_NULL,
        related_name='+',
        verbose_name="page des mentions légales",
    )
    credits = models.CharField(
        "crédits",
        max_length=255,
        blank=True,
        help_text=mark_safe(
            "Un format court des crédits du site, affichés en bas de chaque "
            "page. À noter : la France ne faisant pas partie du <i>Common "
            "law</i>, la mention <i>Copyright</i> est inutile."
        ),
    )

    # Suivi et confidentialité des données
    privacy_policy = RichTextField(
        "politique de confidentialité",
        blank=True,
        features=(
            'h3',
            'h4',
            'bold',
            'italic',
            'ol',
            'ul',
            'hr',
            'link',
            'document-link',
        ),
        help_text=mark_safe(
            "Texte expliquant aux visiteurs les données qui sont collectées "
            "par le site et comment elles sont utilisées. Il est affiché sur "
            "la page <i>Politique de confidentialité</i>, qui permet aussi "
            "de s'opposer au suivi si le code est défini."
        ),
    )
    tracking_code = models.TextField(
        "code de suivi",
        blank=True,
        help_text=mark_safe(
            "Le code JavaScript à exécuter pour suivre les visiteurs sur le "
            "site, sans la balise <code>&lt;script&gt;</code>."
        ),
    )

    # Attributs Wagtails
    # ------------------

    # Panneaux pour l'administration
    contact_panels = [
        MultiFieldPanel(
            [
                FieldPanel('phone'),
                FieldPanel('address'),
                FieldPanel('opening_hours'),
                FieldPanel('contact_link'),
                FieldPanel('map_link'),
            ],
            heading="Informations pratiques",
        ),
        MultiFieldPanel(
            [
                FieldPanel('facebook_link'),
                FieldPanel('instagram_link'),
                FieldPanel('youtube_link'),
            ],
            heading="Liens vers les réseaux",
        ),
    ]
    legacy_panels = [
        MultiFieldPanel(
            [PageChooserPanel('legal_notice_page'), FieldPanel('credits')],
            heading="Informations légales",
        ),
        MultiFieldPanel(
            [FieldPanel('privacy_policy'), FieldPanel('tracking_code')],
            heading="Suivi et confidentialité",
        ),
    ]

    edit_handler = TabbedInterface(
        [
            ObjectList(contact_panels, heading="Contacts"),
            ObjectList(legacy_panels, heading="Mentions et suivi"),
        ]
    )

    class Meta:
        verbose_name = "informations sur la ville"

    def __str__(self):
        if hasattr(self, 'site') and self.site.site_name:
            return self.site.site_name
        return getattr(settings, 'WAGTAIL_SITE_NAME', "Site par défaut")

    @cached_property
    def networks_links(self):
        """Retourne un dictionnaire des réseaux définis avec leur lien."""
        networks_links = OrderedDict()
        for network in ('facebook', 'instagram', 'youtube'):
            link = getattr(self, '{}_link'.format(network))
            if link:
                networks_links[network] = link
        return networks_links

    @property
    def has_privacy_policy(self):
        return self.privacy_policy or self.tracking_code

    def clean(self):
        if self.tracking_code and self.tracking_code.strip()[0] == '<':
            raise ValidationError(
                {
                    'tracking_code': (
                        "Le code JavaScript semble invalide, il ne devrait "
                        "pas commencer par « < »."
                    )
                }
            )


# MODELES DES PAGES
# ------------------------------------------------------------------------------


class AbstractHeaderPage(Page):
    """
    Modèle abstrait d'une page avec un en-tête.
    """

    introduction = models.TextField(
        "chapeau",
        max_length=255,
        blank=True,
        help_text=(
            "Un texte bref introduisant la page, affiché en dessous du titre."
        ),
    )
    banner = models.ForeignKey(
        'wagtailimages.Image',
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name="+",
        verbose_name="bannière",
        help_text=(
            "Une image utilisée comme bannière de fond pour le titre. Veillez "
            "à ce que la résolution soit assez grande pour garantir "
            "la qualité d'affichage."
        ),
    )

    BANNER_BREAKPOINTS = [576, 768, 1200, 1600, 1920]
    BANNER_RENDITION = 'fill-{width}x400-c75'

    # Attributs Wagtails
    # ------------------

    # Panneaux pour l'administration
    content_panels = [
        MultiFieldPanel(
            [
                FieldPanel('title'),
                FieldPanel('introduction'),
                ImageChooserPanel('banner'),
            ],
            heading="En-tête",
        )
    ]

    # Champs de recherche
    search_fields = Page.search_fields + [index.SearchField('introduction')]

    class Meta:
        abstract = True

    def get_banner_renditions(self):
        """
        Retourne une liste de variantes de la bannière à utiliser pour les
        différentes résolutions, en fonction de `BANNER_BREAKPOINTS`. Une
        variante est un tuple de deux éléments : la largeur minimale à
        laquelle s'applique l'image et l'image.
        """
        if not self.banner:
            return []
        return [
            (
                min_width,
                self.banner.get_rendition(
                    self.BANNER_RENDITION.format(width=width)
                ),
            )
            for min_width, width in zip(
                [0, *self.BANNER_BREAKPOINTS[:-1]], self.BANNER_BREAKPOINTS
            )
        ]

    def get_context(self, request):
        context = super().get_context(request)
        context['banner_renditions'] = self.get_banner_renditions()
        return context


class AbstractBodyPage(AbstractHeaderPage):
    """
    Modèle abstrait d'une page avec du contenu.
    """

    body = StreamField(
        BodyStreamBlock(), blank=True, verbose_name="contenu de la page"
    )

    # Attributs Wagtails
    # ------------------

    # Panneaux pour l'administration
    content_panels = AbstractHeaderPage.content_panels + [
        StreamFieldPanel('body')
    ]

    # Champs de recherche
    search_fields = AbstractHeaderPage.search_fields + [
        index.SearchField('body')
    ]

    class Meta:
        abstract = True


class StandardPage(AbstractBodyPage):
    """
    Une page standard avec du contenu.
    """

    has_menu = models.BooleanField(
        "menu de navigation rapide",
        default=False,
        help_text=(
            "Génère et affiche un menu composé des titres de niveau 3, "
            "permettant de naviguer au sein de la page. Utile pour les "
            "longues pages."
        ),
    )

    # Attributs Wagtails
    # ------------------

    show_in_menus_default = True

    # Panneaux pour l'administration
    settings_panels = Page.settings_panels + [
        MultiFieldPanel([FieldPanel('has_menu')], heading="Contenu")
    ]

    class Meta:
        verbose_name = "page standard"
        verbose_name_plural = "pages standards"


class CategoryPage(AbstractHeaderPage):
    """
    Une rubrique contenant d'autres pages.
    """

    # Attributs Wagtails
    # ------------------

    show_in_menus_default = True

    class Meta:
        verbose_name = "rubrique"
        verbose_name_plural = "rubriques"

    def get_context(self, request):
        context = super().get_context(request)
        context['child_pages'] = self.get_children().live()
        return context


class LinkPage(AbstractLinkPage):
    """
    Un lien vers une page ou une URL.
    """

    class Meta:
        verbose_name = "lien"
        verbose_name_plural = "liens"


class HomePage(Page):
    """
    La page d'accueil du site.
    """

    # En-tête de la page, avec une image d'arrière-plan et des boutons
    hero_banner = models.ForeignKey(
        'wagtailimages.Image',
        null=True,
        on_delete=models.SET_NULL,
        related_name="+",
        verbose_name="bannière",
        help_text=(
            "Une image utilisée comme arrière-plan de la section. Veillez "
            "à ce que la résolution soit assez grande pour garantir "
            "la qualité d'affichage."
        ),
    )
    hero_links = StreamField(
        StreamBlock([('button', ButtonBlock())]),
        null=True,
        verbose_name="liens",
        help_text=(
            "Ces liens seront présentés par-dessus la bannière. Ils peuvent "
            "pointer vers des pages internes ou des liens externes."
        ),
    )
    hero_links_mobile = StreamField(
        StreamBlock([('button', ButtonBlock())]),
        null=True,
        verbose_name="liens (mobile)",
        help_text=(
            "Ces liens seront présentés par-dessus la bannière, sur mobile "
            "uniquement à la place des liens définis ci-dessus."
        ),
    )

    body = StreamField(
        HomeStreamBlock(), blank=True, verbose_name="sections de la page"
    )

    # Attributs Wagtails
    # ------------------

    # N'autorise cette page qu'à la racine
    parent_page_types = ['wagtailcore.Page']

    # Panneaux pour l'administration
    content_panels = [
        FieldPanel('title'),
        MultiFieldPanel(
            [
                ImageChooserPanel('hero_banner'),
                StreamFieldPanel('hero_links'),
                StreamFieldPanel('hero_links_mobile'),
            ],
            heading="En-tête",
        ),
        StreamFieldPanel('body'),
    ]

    class Meta:
        verbose_name = "page d'accueil"
        verbose_name_plural = "pages d'accueil"
