from django.core.exceptions import ValidationError

import pytest
from wagtail_factories import ImageFactory

from lens.base import blocks
from lens.utils.tests import BlockTest


class TestRowBlock:
    @pytest.mark.parametrize(
        'size, breakpoint, expected',
        [
            ('', None, ''),
            ('unknown', None, ''),
            ('0', None, 'col'),
            ('1', None, 'col-12'),
            ('2', None, 'col-6'),
            ('3', None, 'col-4'),
            ('4', None, 'col-3'),
            ('6', None, 'col-2'),
            ('12', None, 'col-1'),
            ('auto', None, 'col-auto'),
            ('auto', 'sm', 'col-sm-auto'),
        ],
    )
    def test_grid_column_size_css_class(self, size, breakpoint, expected):
        assert (
            blocks.Grid.get_column_size_css_class(size, breakpoint) == expected
        )

    @pytest.mark.parametrize(
        'nb_sm, nb_md, nb_lg, halign, expected',
        [
            ('', '', '', '', 'col'),
            ('0', '1', 'auto', '', 'col-sm col-md-12 col-lg-auto'),
            ('', '1', 'auto', '', 'col-md-12 col-lg-auto'),
            ('0', '', 'auto', 'left', 'col-sm col-lg-auto text-left'),
            ('0', '', '', 'center', 'col-sm text-center'),
            ('0', '', '', 'right', 'col-sm text-right'),
        ],
    )
    def test_get_context_column_css_class(
        self, nb_sm, nb_md, nb_lg, halign, expected
    ):
        block = blocks.RowBlock()
        context = block.get_context(
            block.to_python(
                {
                    'nb_columns_sm': nb_sm,
                    'nb_columns_md': nb_md,
                    'nb_columns_lg': nb_lg,
                    'horizontal_align': halign,
                    'blocks': [],
                }
            )
        )
        assert context['column_css_class'] == expected

    @pytest.mark.parametrize(
        'align, expected',
        [
            ('', ''),
            ('unknown', ''),
            ('left', 'justify-content-start'),
            ('center', 'justify-content-center'),
            ('right', 'justify-content-end'),
        ],
    )
    def test_get_context_row_css_class(self, align, expected):
        block = blocks.RowBlock()
        context = block.get_context(
            block.to_python(
                {
                    'nb_columns_sm': '',
                    'nb_columns_md': '',
                    'nb_columns_lg': '',
                    'horizontal_align': align,
                    'blocks': [],
                }
            )
        )
        assert context['row_css_class'] == expected

    def test_block_hidden(self):
        block = blocks.RowBlock()
        data = {
            'nb_columns_sm': '',
            'nb_columns_md': '',
            'nb_columns_lg': '',
            'horizontal_align': '',
            'blocks': [],
            'hidden': True,
        }
        assert block.render(data) == ''


class TestTitleBlock(BlockTest):
    block = blocks.TitleBlock()

    @pytest.mark.parametrize(
        'title, level, hidden, exp',
        [
            (
                'Un titre !',
                'h3',
                False,
                '<h3 class="title" id="un-titre">Un titre !</h3>',
            ),
            ('Un titre !', 'h3', True, ''),
        ],
    )
    def test_render(self, title, level, hidden, exp):
        assert (
            self.render({'title': title, 'level': level, 'hidden': hidden})
            == exp
        )


class TestParagraphBlock(BlockTest):
    block = blocks.ParagraphBlock()

    def test_features(self):
        assert self.block.features == blocks.ParagraphBlock.features

        block = blocks.ParagraphBlock(features=['bold', 'italic'])
        assert block.features == ['bold', 'italic']

    def test_render(self):
        assert (
            self.render('<i>Un</i> paragraphe !')
            == '<div class="rich-text"><i>Un</i> paragraphe !</div>\n'
        )


class TestQuoteBlock(BlockTest):
    block = blocks.QuoteBlock()

    def test_render(self):
        data = {'text': '<i>Un</i> paragraphe !', 'author': 'Proudhon'}
        assert self.render(data) == (
            '<blockquote class="blockquote">\n'
            '  <div class="rich-text"><i>Un</i> paragraphe !</div>\n'
            '  \n'
            '  <footer class="blockquote-footer">\n'
            '    <cite>Proudhon</cite>\n'
            '  </footer>\n'
            '  \n'
            '</blockquote>\n'
        )

    def test_render_when_hidden(self):
        data = {
            'text': '<i>Un</i> paragraphe !',
            'author': 'Proudhon',
            'hidden': True,
        }
        assert self.render(data) == ''


@pytest.mark.django_db
class TestButtonBlock(BlockTest):
    block = blocks.ButtonBlock()

    @pytest.mark.parametrize(
        "link, href_result, size",
        [
            (
                {'external': 'https://kernel.org'},
                'https://kernel.org',
                'btn-small',
            ),
            ({'anchor': 'mon_ancre'}, '#mon_ancre', 'btn-medium'),
            ({'page': 2}, 'http://localhost/', 'btn-large'),
            ({'document': 1}, '/documents/1/doc.pdf', ""),
        ],
    )
    def test_render(self, document, link, href_result, size):
        data = {'text': 'Mon beau bouton', 'link': link, 'size': size}
        if link.get('external') or link.get('document'):
            if size == "":
                assert self.render(data) == (
                    '<div class="btn-holder">\n'
                    '  <a href="{}" class="btn btn-arrow-right" '
                    'target="_blank">'
                    'Mon beau bouton</a>\n'
                    '</div>'.format(href_result)
                )
            else:
                assert self.render(data) == (
                    '<div class="btn-holder">\n'
                    '  <a href="{}" class="btn btn-arrow-right {}" '
                    'target="_blank">'
                    'Mon beau bouton</a>\n'
                    '</div>'.format(href_result, size)
                )
        else:
            assert self.render(data) == (
                '<div class="btn-holder">\n'
                '  <a href="{}" class="btn btn-arrow-right {}">'
                'Mon beau bouton</a>\n'
                '</div>'.format(href_result, size)
            )

    @pytest.mark.parametrize(
        "link, hidden",
        [
            ({'external': 'https://kernel.org'}, True),
            ({'anchor': 'mon_ancre'}, True),
            ({'page': 2}, True),
            ({'document': 1}, True),
        ],
    )
    def test_render_when_hidden(self, link, hidden):
        data = {'text': 'Mon beau bouton', 'link': link, 'hidden': hidden}
        assert self.render(data) == ''


class TestLinkBlock:
    block = blocks.LinkBlock()

    def test_no_target(self):
        with pytest.raises(ValidationError) as err:
            self.block.clean({})
        assert (
            err.value.args[2]['page'][0]
            == 'Une cible est requise pour ce lien.'
        )

    def test_anchor_and_incompatible_target(self):
        with pytest.raises(ValidationError) as err:
            self.block.clean(
                {'external': 'https://kernel.org', 'anchor': 'anchor'}
            )
        assert (
            err.value.args[2]['anchor'][0]
            == "L'ancre n'est que valable seule ou avec une page."
        )

    @pytest.mark.django_db
    def test_more_than_one_target(self):
        with pytest.raises(ValidationError) as err:
            self.block.clean({'external': 'https://kernel.org', 'page': 2})
        assert (
            err.value.args[2]['external'][0]
            == "Le lien ne peut pointer que vers une cible."
        )

    def test_good_target(self):
        data = {'external': 'https://kernel.org'}
        assert isinstance(self.block.clean(data), blocks.LinkStructValue)


@pytest.mark.django_db
class TestPDFBlock(BlockTest):
    block = blocks.PDFBlock()

    def test_render(self, document):
        html = self.render({'file': 1})
        assert html == (
            '<object height="500px" width="100%" data="/documents/1/doc.pdf">'
            '</object>'
        )

    def test_render_when_hidden(self, document):
        html = self.render({'file': 1, 'hidden': True})
        assert html == ''


@pytest.mark.django_db
class TestImageBlock(BlockTest):
    block = blocks.ImageBlock()

    def test_render(self):
        html = self.render_html(
            {'image': ImageFactory().id, 'caption': '', 'link': {'target': []}}
        )
        assert len(html.select('img.figure-img.img-fluid')) == 1
        assert not html.select('figcaption')
        assert not html.select('a')

    def test_render_with_caption(self):
        html = self.render_html(
            {
                'image': ImageFactory().id,
                'caption': "Une légende en dessous.",
                'link': {'target': []},
            }
        )
        assert html.select_one('figcaption').text == "Une légende en dessous."
        assert not html.select('a')

    def test_render_with_link(self):
        url = 'http://example.org/truc/'
        html = self.render_html(
            {
                'image': ImageFactory().id,
                'caption': '',
                'link': {'external': url},
            }
        )
        assert html.select_one('a').attrs['href'] == url
        assert not html.select('figcaption')

    @pytest.mark.parametrize(
        'caption, link',
        [
            ('', {'target': []}),
            ('légende', {'target': []}),
            ('', {'external': 'http://example.org/truc/'}),
        ],
    )
    def test_render_when_hidden(self, caption, link):
        data = {
            'image': ImageFactory().id,
            'caption': caption,
            'link': link,
            'hidden': True,
        }
        assert self.render(data) == ''


@pytest.mark.django_db
class TestImageTextBlock(BlockTest):
    block = blocks.ImageTextBlock()

    def test_render(self):
        html = self.render_html(
            {
                'image': ImageFactory().id,
                'text': "Texte sur l'image",
                'is_animated': True,
                'caption': '',
                'link': {'target': []},
            }
        )
        assert 'is-animated' in html.select('figure div')[0]['class']
        assert (
            html.select('.figure-text > .rich-text')[0].string
            == "Texte sur l'image"
        )

    def test_render_when_hidden(self):
        data = {
            'image': ImageFactory().id,
            'text': "Texte sur l'image",
            'is_animated': True,
            'caption': '',
            'link': {'target': []},
            'hidden': True,
        }
        assert self.render(data) == ''


@pytest.mark.django_db
class TestImageCarouselBlock(BlockTest):
    block = blocks.ImageCarouselBlock()

    def test_render(self):
        image1 = ImageFactory()
        image2 = ImageFactory()
        html = self.render_html(
            {
                'items': [
                    {
                        'type': 'image',
                        'value': {'image': image1.id, 'caption': ''},
                    },
                    {
                        'type': 'image',
                        'value': {'image': image2.id, 'caption': ''},
                    },
                ]
            }
        )
        assert len(html.select('.tns-slider-items')) == 1
        assert len(html.select('img')) == 2
        assert (
            html.select_one('.figure-img').attrs['src']
            == image1.get_rendition('max-600x600').url
        )

    def test_render_when_hidden(self):
        image1 = ImageFactory()
        image2 = ImageFactory()
        data = {
            'items': [
                {
                    'type': 'image',
                    'value': {'image': image1.id, 'caption': ''},
                },
                {
                    'type': 'image',
                    'value': {'image': image2.id, 'caption': ''},
                },
            ],
            'hidden': True,
        }
        assert self.render(data) == ''


class TestIconTextBlock(BlockTest):
    block = blocks.IconTextBlock()
    tel = '06 00 00 00 00'

    def test_render(self):
        html = self.render_html({'icon': 'phone', 'text': self.tel})
        assert html.select_one('.icon-phone')
        assert html.select_one('figcaption').string == self.tel

    def test_render_when_hidden(self):
        data = {'icon': 'phone', 'text': self.tel, 'hidden': True}
        assert self.render(data) == ''


class TestColumnBlock:
    @pytest.mark.parametrize(
        'size, breakpoint, expected',
        [
            ('', None, ''),
            ('unknown', None, ''),
            ('0', None, 'col'),
            ('1', None, 'col-12'),
            ('2', None, 'col-6'),
            ('3', None, 'col-4'),
            ('4', None, 'col-3'),
            ('6', None, 'col-2'),
            ('12', None, 'col-1'),
            ('auto', None, 'col-auto'),
            ('auto', 'sm', 'col-sm-auto'),
        ],
    )
    def test_grid_column_size_css_class(self, size, breakpoint, expected):
        assert (
            blocks.Grid.get_column_size_css_class(size, breakpoint) == expected
        )

    @pytest.mark.parametrize(
        'size_sm, size_md, size_lg, halign, expected',
        [
            ('', '', '', '', 'col'),
            ('0', '1', 'auto', '', 'col-sm col-md-12 col-lg-auto'),
            ('', '1', 'auto', '', 'col-md-12 col-lg-auto'),
            ('0', '', 'auto', 'left', 'col-sm col-lg-auto text-left'),
            ('0', '', '', 'center', 'col-sm text-center'),
            ('0', '', '', 'right', 'col-sm text-right'),
        ],
    )
    def test_get_context_column_css_class(
        self, size_sm, size_md, size_lg, halign, expected
    ):
        block = blocks.ColumnBlock()
        context = block.get_context(
            block.to_python(
                {
                    'size_sm': size_sm,
                    'size_md': size_md,
                    'size_lg': size_lg,
                    'horizontal_align': halign,
                    'block': [],
                }
            )
        )
        assert context['column_css_class'] == expected

    @pytest.mark.parametrize(
        'align, expected',
        [
            ('', 'col'),
            ('unknown', 'col'),
            ('left', 'col text-left'),
            ('center', 'col text-center'),
            ('right', 'col text-right'),
        ],
    )
    def test_get_context_column_css__col_class(self, align, expected):
        block = blocks.ColumnBlock()
        context = block.get_context(
            block.to_python(
                {
                    'size_sm': '',
                    'size_md': '',
                    'size_lg': '',
                    'horizontal_align': align,
                    'block': [],
                }
            )
        )
        assert context['column_css_class'] == expected

    def test_block_hidden(self):
        block = blocks.ColumnBlock()
        data = {
            'size_sm': '',
            'size_md': '',
            'size_lg': '',
            'horizontal_align': '',
            'block': [],
            'hidden': True,
        }
        assert block.render(data) == ''


@pytest.mark.django_db
class TestSectionBlock(BlockTest):
    block = blocks.SectionBlock()

    def test_render(self, events):  # noqa: F811
        data = {
            'title': 'Section evenements',
            'content': [
                {
                    'type': 'upcoming_events',
                    'value': {
                        'limit': 2,
                        'add_all_link': True,
                    },
                }
            ],
        }
        html = self.render(data)
        assert '<h3 class="title">Section evenements</h3>' in html
        assert 'Event 1' in html
        assert 'Event 2' in html

    def test_render_when_hidden(self, events):  # noqa: F811
        data = {
            'title': 'Section evenements',
            'content': [
                {
                    'type': 'upcoming_events',
                    'value': {
                        'limit': 2,
                        'add_all_link': True,
                    },
                }
            ],
            'hidden': True,
        }
        assert self.render(data) == ''
