import os.path

from django.core.exceptions import ValidationError
from django.core.files import File
from django.core.files.base import ContentFile

import pytest
from wagtail.images.tests.utils import Image, get_test_image_file

from ..models import OrganizationInfo, SourceMedia, StandardPage

TRACKING_CODE = """
var _paq = window._paq || [];
_paq.push(['trackPageView']);
_paq.push(['enableLinkTracking']);
(function() {
  var u="https://matomo.example.org/";
  _paq.push(['setTrackerUrl', u+'matomo.php']);
  _paq.push(['setSiteId', '1']);
  var d=document, g=d.createElement('script'),
      s=d.getElementsByTagName('script')[0];
  g.type='text/javascript'; g.async=true; g.defer=true; g.src=u+'matomo.js';
  s.parentNode.insertBefore(g,s);
})();
"""

FULL_TRACKING_CODE = """
<!-- Matomo -->
<script type="text/javascript">{}</script>
<!-- End Matomo Code -->
""".format(
    TRACKING_CODE
)

AUDIO_SINE_PATH = os.path.join(os.path.dirname(__file__), 'data/sine.ogg')


@pytest.fixture
def media_audio():
    return SourceMedia(
        title="Audio test",
        type='audio',
        file=File(open(AUDIO_SINE_PATH, 'rb')),
    )


@pytest.mark.django_db
class TestSourceMedia:
    def test_source_tag(self, media_audio):
        assert media_audio.source_tag() == (
            '<source src="{}">'.format(media_audio.url)
        )

        assert media_audio.source_tag(extra_attributes={'foo': 'bar'}) == (
            '<source foo="bar" src="{}">'.format(media_audio.url)
        )

        media_audio.mimetype = 'audio/ogg'
        assert media_audio.source_tag() == (
            '<source src="{}" type="audio/ogg">'.format(media_audio.url)
        )

    def test_source_tag_empty(self):
        assert SourceMedia().source_tag() == '<source>'

    def test_clean_file(self, media_audio):
        assert media_audio.mimetype is None
        media_audio.clean()
        assert media_audio.mimetype.startswith('audio/')

    def test_clean_file_empty(self):
        media = SourceMedia()
        media.clean()
        assert media.mimetype is None

        media.type = 'audio'
        media.clean()
        assert media.mimetype is None

    def test_clean_file_invalid_mimetype(self):
        media = SourceMedia(
            title="Audio test",
            type='audio',
            file=ContentFile("", name='file.txt'),
        )
        with pytest.raises(ValidationError) as excinfo:
            media.clean()
        assert excinfo.value.error_dict['file'][0].code == 'invalid_mimetype'

    def test_clean_file_invalid(self):
        media = SourceMedia(
            title="Audio test",
            type='audio',
            file="some unexpected file object",
        )
        with pytest.raises(ValidationError) as excinfo:
            media.clean()
        assert excinfo.value.error_dict['file'][0].code == 'invalid'


class TestOrganizationInfo:
    def test_clean(self):
        org_info = OrganizationInfo()
        org_info.clean()

        org_info.tracking_code = TRACKING_CODE
        org_info.clean()

    @pytest.mark.parametrize(
        'value',
        (
            FULL_TRACKING_CODE,
            '<script type="text/javascript">{}</script>'.format(TRACKING_CODE),
        ),
    )
    def test_clean_error(self, value):
        org_info = OrganizationInfo(tracking_code=value)
        with pytest.raises(ValidationError):
            org_info.clean()


class TestAbstractHeaderPage:
    def test_banner_renditions_empty(self):
        page = StandardPage()
        assert page.get_banner_renditions() == []

    @pytest.mark.django_db
    def test_banner_renditions(self):
        page = StandardPage()
        page.banner = Image.objects.create(
            title="Banner", file=get_test_image_file(size=(1920, 800))
        )
        renditions = page.get_banner_renditions()
        assert len(renditions) == 5  # len(page.BANNER_BREAKPOINTS)
        assert [min_width for min_width, image in renditions] == [
            0,
            576,
            768,
            1200,
            1600,
        ]
        assert renditions[0][1].width == 576
