from django.templatetags.static import static
from django.utils.html import format_html

import wagtail.admin.rich_text.editors.draftail.features as draftail_features
from wagtail.admin.menu import MenuItem
from wagtail.admin.rich_text.converters.html_to_contentstate import (
    InlineStyleElementHandler,
)
from wagtail.core import hooks

from .models import CategoryPage
from .templatetags.base_tags import min_static


class SiteMenuItem(MenuItem):
    """
    Élément de menu pour retourner vers le site.
    """

    def __init__(self, label="Retour au site", url='', **kwargs):
        super().__init__(label, '', **kwargs)

    def is_active(self, *args, **kwargs):
        return False

    def get_context(self, request):
        context = super().get_context(request)
        context['url'] = request.META['SCRIPT_NAME'] or '/'
        return context


@hooks.register('register_admin_menu_item')
def register_site_menu_item():
    """Ajoute un élément de menu pour retourner vers le site."""
    return SiteMenuItem(classnames='icon icon-home', order=10000)


@hooks.register('insert_global_admin_css')
def global_admin_css():
    return format_html(
        '<link rel="stylesheet" href="{}">', min_static('css/admin.css')
    )


@hooks.register('menus_modify_primed_menu_items')
def modify_primed_menu_items(menu_items, menu_tag, **kwargs):
    """Supprime le lien des pages de type rubrique."""
    if menu_tag == 'sub_menu':
        for item in menu_items:
            if item.specific_class == CategoryPage:
                item.href = '#'
    else:
        for item in menu_items:
            if (
                getattr(item, 'link_page')
                and item.link_page.specific_class == CategoryPage
            ):
                item.href = '#'
    return menu_items


@hooks.register('insert_editor_js')
def editor_js():
    return format_html(
        '<script src="{}"></script>', static('js/toggle_editor_block.js')
    )


@hooks.register('register_rich_text_features')
def register_coloring_features(features):
    """Ajoute la possibilité d'écrire du texte en bleu/rouge dans l'éditeur"""

    square_path = ['M100 100 H 900 V 900 H 100 Z']
    tag = 'span'

    # draftail a besoin des couleurs hard-codées pour l'affichage dans la zone
    # de texte
    colors = {'primary': '#bd003f', 'secondary': '#00aeef'}

    for color, color_hex in colors.items():
        css_class = 'text-{}'.format(color)
        feature_name = 'color-{}'.format(color)
        type_ = feature_name

        features.register_editor_plugin(
            'draftail',
            feature_name,
            draftail_features.InlineStyleFeature(
                {
                    'type': type_,
                    'icon': square_path,
                    'style': {'color': color_hex},
                }
            ),
        )
        db_conversion = {
            'from_database_format': {
                '{}[class={}]'.format(
                    tag, css_class
                ): InlineStyleElementHandler(type_)
            },
            'to_database_format': {
                'style_map': {
                    type_: {
                        'element': tag,
                        'props': {'class': css_class},
                    }
                }
            },
        }
        features.register_converter_rule(
            'contentstate', feature_name, db_conversion
        )
        features.default_features.append(feature_name)
