import mimetypes

from django.contrib.syndication.views import Feed

from bs4 import BeautifulSoup

from .models import BlogArticlePage, BlogCategoryPage, FeedSettings


class LatestArticlesFeed(Feed):
    title = "Derniers articles de la ville de Lens"
    link = '/'
    description = (
        "Ce flux contient les derniers articles publiés sur le site "
        "officiel de la ville de Lens."
    )

    def get_feed(self, obj, request):
        if getattr(request, 'site') is None:
            raise ValueError("Aucun site n'est défini pour la requête.")
        self.request = request
        self.settings = FeedSettings.for_site(request.site)
        return super().get_feed(obj, request)

    def items(self):
        queryset = BlogArticlePage.objects.live().public()
        if self.settings.categories:
            # récupère toutes les rubriques en une seule requête
            categories = list(
                BlogCategoryPage.objects.filter(
                    pk__in=[
                        data['value']
                        for data in self.settings.categories.raw_data
                    ]
                )
            )
            # construit la combinaison de filtres avec les rubriques
            filter_qs = queryset.child_of_q(categories[0])
            for category in categories[1:]:
                filter_qs |= queryset.child_of_q(category)
            queryset = queryset.filter(filter_qs)
        return queryset.order_by('-first_published_at')[: self.settings.limit]

    def item_title(self, item):
        return item.title

    def item_description(self, item):
        exclude_blocks = ['embed_block', 'button_block']
        content = [
            streamfield.render()
            for streamfield in item.body
            if streamfield.block_type not in exclude_blocks
        ]
        return self.fix_relative_url('\n'.join(content))

    def item_link(self, item):
        return item.get_full_url()

    def item_enclosure_url(self, item):
        if not item.banner:
            return
        return self.request.build_absolute_uri(item.banner.file.url)

    def item_enclosure_length(self, item):
        return item.banner.file.size

    def item_enclosure_mime_type(self, item):
        return mimetypes.guess_type(item.banner.file.name)[0] or ''

    def item_pubdate(self, item):
        return item.first_published_at

    def fix_relative_url(self, html):
        """
        Transforme les URL relatives en absolues.

        Les éléments d'un flux RSS peuvent contenir du code HTML, comme des
        images ou des liens, mais les URL ne doivent pas être relatives :

          https://validator.w3.org/feed/docs/warning/ContainsRelRef.html
        """
        soup = BeautifulSoup(html, 'html.parser')
        for tag in soup.find_all('a'):
            if 'href' in tag.attrs and tag['href'] != '#':
                tag['href'] = self.request.build_absolute_uri(tag['href'])
            else:  # pragma: no cover
                pass
        for tag in soup.find_all('img'):
            if 'src' in tag.attrs:
                tag['src'] = self.request.build_absolute_uri(tag['src'])
            else:  # pragma: no cover
                pass
        return soup
