from django.conf import settings
from django.core.validators import MinValueValidator
from django.db import models
from django.db.models.functions import Coalesce

from modelcluster.contrib.taggit import ClusterTaggableManager
from modelcluster.fields import ParentalKey
from taggit.models import TaggedItemBase
from wagtail.admin.edit_handlers import (
    FieldPanel,
    MultiFieldPanel,
    StreamFieldPanel,
)
from wagtail.contrib.settings.models import BaseSetting, register_setting
from wagtail.core.blocks import PageChooserBlock
from wagtail.core.fields import StreamField
from wagtail.images.edit_handlers import ImageChooserPanel

from lens.base.models import AbstractBodyPage, AbstractHeaderPage


@register_setting(icon='code')
class FeedSettings(BaseSetting):
    """
    Configuration du flux RSS des articles.
    """

    categories = StreamField(
        [
            (
                'category',
                PageChooserBlock(
                    # TODO: renommer en page_type à partir de Wagtail 2.5
                    target_model='blog.BlogCategoryPage',
                    label="Rubrique",
                    icon='folder',
                ),
            )
        ],
        blank=True,
        null=True,
        verbose_name="rubriques",
        help_text=(
            "Seuls les articles de cette ou ces rubriques seront retournés "
            "par le flux. Laissez vide pour ne pas filtrer sur la rubrique "
            "et retourner tous les articles."
        ),
    )
    limit = models.PositiveSmallIntegerField(
        "limite",
        default=10,
        validators=[MinValueValidator(1)],
        help_text="Nombre maximal d'articles contenus dans le flux.",
    )

    # Attributs Wagtails
    # ------------------

    panels = [
        MultiFieldPanel(
            [StreamFieldPanel('categories'), FieldPanel('limit')],
            heading="Contenu du flux",
        )
    ]

    class Meta:
        verbose_name = "flux RSS"

    def __str__(self):
        return self.site.site_name or getattr(settings, 'WAGTAIL_SITE_NAME')


# MODELES DES PAGES
# ------------------------------------------------------------------------------


class BlogArticlePageTag(TaggedItemBase):
    """
    Relation intermédiaire entre un article et une étiquette.
    """

    content_object = ParentalKey(
        'BlogArticlePage',
        on_delete=models.CASCADE,
        related_name='tagged_items',
    )


class BlogArticlePage(AbstractBodyPage):
    """
    Un article de blog.
    """

    tags = ClusterTaggableManager(
        through=BlogArticlePageTag,
        blank=True,
        verbose_name="étiquettes",
        help_text=(
            "Une liste d'une ou plusieurs étiquettes séparées par une "
            "virgule."
        ),
    )

    # Attributs Wagtails
    # ------------------

    # Restreint la page parente à une rubrique d'articles
    parent_page_types = ['BlogCategoryPage']

    # Empêche les pages enfants
    subpage_types = []

    # Panneaux pour l'administration
    content_panels = [
        MultiFieldPanel(
            [
                FieldPanel('title'),
                FieldPanel('tags'),
                ImageChooserPanel('banner'),
            ],
            heading="En-tête",
        ),
        StreamFieldPanel('body'),
    ]

    class Meta:
        verbose_name = "article"
        verbose_name_plural = "articles"

    @property
    def category(self):
        return self.get_parent()


class BlogCategoryPage(AbstractHeaderPage):
    """
    Une rubrique de blog contenant des articles.
    """

    # Attributs Wagtails
    # ------------------

    # Restreint les pages enfants aux articles
    subpage_types = ['BlogArticlePage']

    class Meta:
        verbose_name = "rubrique d'articles"
        verbose_name_plural = "rubriques d'articles"

    @property
    def articles(self):
        """Retourne les articles de la rubrique, triés par date."""
        return (
            BlogArticlePage.objects.descendant_of(self)
            .live()
            .order_by(Coalesce('go_live_at', 'first_published_at').desc())
        )

    def get_context(self, request):
        context = super().get_context(request)
        context['articles'] = self.articles
        return context
