import re
from xml.dom import minidom

from django.urls import reverse_lazy

import pytest
from wagtail.core.models import Site

from lens.blog.models import BlogArticlePage, BlogCategoryPage, FeedSettings


def get_items(response):
    doc = minidom.parseString(response.content)
    return doc.getElementsByTagName('item')


def get_tag_value(parent_elem, name):
    elem = parent_elem.getElementsByTagName(name)
    return elem[0].firstChild.wholeText


@pytest.mark.django_db
@pytest.mark.usefixtures('with_blog_site')
class TestLatestArticlesFeed:
    url = reverse_lazy('feed')

    def test_no_site(self, client):
        site = Site.objects.get(is_default_site=True)
        site.is_default_site = False
        site.save()

        with pytest.raises(ValueError):
            client.get(self.url)

    def test_response(self, client):
        response = client.get(self.url)
        assert response.status_code == 200

        doc = minidom.parseString(response.content)
        feed_elem = doc.getElementsByTagName('rss')
        assert len(feed_elem) == 1

        items = feed_elem[0].getElementsByTagName('item')
        assert len(items) == 3

    def test_item(self, client):
        article = BlogArticlePage.objects.get(pk=7)
        items = get_items(client.get(self.url))

        assert get_tag_value(items[0], 'title') == article.title
        assert get_tag_value(items[0], 'link') == article.get_full_url()

        description = get_tag_value(items[0], 'description')
        # les boutons sont exclus de la description
        assert 'Bouton' in article.body.render_as_block()
        assert 'Bouton' not in description
        # la cible des liens et des images est absolue
        assert 'href="http://example.org/"' in description
        assert re.search(r'href="http://.+/offres-demplois/"', description)
        assert re.search(r'src="http://.+/bar[\._].+\.jpg"', description)

        # la balise <enclosure> contient la bannière si définie
        assert not items[1].getElementsByTagName('enclosure')
        enclosure = items[0].getElementsByTagName('enclosure')[0]
        assert enclosure.getAttribute('type') == 'image/jpeg'
        assert enclosure.getAttribute('url').startswith('http://')

    def test_filter_categories(self, client):
        categories = list(BlogCategoryPage.objects.all().order_by('pk'))
        settings = FeedSettings.objects.get(pk=1)

        settings.categories = [('category', categories[1])]
        settings.save()
        assert len(get_items(client.get(self.url))) == 1

        settings.categories = [
            ('category', categories[0]),
            ('category', categories[1]),
        ]
        settings.save()
        assert len(get_items(client.get(self.url))) == 3

    def test_limit(self, client):
        settings = FeedSettings.objects.get(pk=1)
        settings.limit = 2
        settings.save()

        items = get_items(client.get(self.url))
        assert len(items) == 2
        assert get_tag_value(items[0], 'title') == "Actualité 2"
        assert get_tag_value(items[1], 'title') == "Emploi 1"

    def test_live_empty(self, client):
        category = BlogCategoryPage.objects.get(pk=5)
        assert category.articles.update(live=False) == 1

        settings = FeedSettings.objects.get(pk=1)
        settings.categories = [('category', category)]
        settings.save()

        assert len(get_items(client.get(self.url))) == 0
