from django import forms
from django.core.exceptions import ValidationError
from django.db import models

from wagtail.admin.edit_handlers import (
    FieldPanel,
    MultiFieldPanel,
    PageChooserPanel,
)

from .validators import validate_svg_file


class Demarche(models.Model):
    """
    Une démarche que le visiteur peut faire en ligne, définie par un nom
    et pointant soit vers un lien externe ou une page du site.
    """

    name = models.CharField("nom", max_length=128, unique=True)
    icon = models.FileField(
        "icône",
        upload_to='demarches',
        validators=[validate_svg_file],
        help_text=(
            "Choisissez un fichier au format SVG, constitué de chemins simples."
        ),
    )
    icon_cache = models.TextField(blank=True, editable=False)

    # Lien de la démarche
    page_link = models.ForeignKey(
        'wagtailcore.Page',
        null=True,
        blank=True,
        on_delete=models.PROTECT,
        related_name='+',
        verbose_name="lien vers une page",
    )
    external_link = models.URLField("lien externe", blank=True)

    # Attributs Wagtails
    # ------------------

    # Panneaux pour l'administration
    panels = [
        MultiFieldPanel(
            [
                FieldPanel('name'),
                FieldPanel(
                    'icon',
                    widget=forms.FileInput(attrs={'accept': 'image/svg+xml'}),
                ),
            ],
            heading="Présentation",
        ),
        MultiFieldPanel(
            [PageChooserPanel('page_link'), FieldPanel('external_link')],
            heading="Lien",
        ),
    ]

    class Meta:
        verbose_name = "démarche en ligne"
        verbose_name_plural = "démarches en ligne"
        ordering = ('name',)

    def __str__(self):
        return self.name

    @property
    def icon_inline(self):
        """Retourne le contenu de l'icône."""
        if not self.icon_cache:
            self.recache_icon()
        return self.icon_cache

    def clean(self, *args, **kwargs):
        """Vérifie qu'un lien est définis."""
        if not self.page_link and not self.external_link:
            msg = "Veuillez choisir une page ou définir un lien externe."
            raise ValidationError({'page_link': msg})
        elif self.page_link and self.external_link:
            msg = "La démarche ne peut pointer que vers un seul lien."
            raise ValidationError({'page_link': msg, 'external_link': msg})

    def save(self, *args, **kwargs):
        super().save(*args, **kwargs)
        self.recache_icon(True)

    def recache_icon(self, save=False):
        """Mets à jour le cache de l'icône et détermine s'il a changé."""
        current_cache = self.icon_cache
        with self.icon.open(mode='r') as stream:
            if stream.tell():
                stream.seek(0)
            new_cache = stream.read()
        if new_cache != current_cache:
            self.icon_cache = new_cache
            if save:
                self.save(update_fields=('icon_cache',))
            return True
        return False
