from django import forms
from django.core.exceptions import ValidationError
from django.db import models
from django.db.models import Q
from django.urls import reverse
from django.utils import timezone
from django.utils.formats import date_format, time_format
from django.utils.functional import cached_property
from django.utils.safestring import mark_safe
from django.utils.text import slugify

from modelcluster.contrib.taggit import ClusterTaggableManager
from modelcluster.fields import ParentalKey
from modelcluster.models import ClusterableModel
from taggit.models import TaggedItemBase
from wagtail.admin.edit_handlers import (
    FieldPanel,
    FieldRowPanel,
    MultiFieldPanel,
    ObjectList,
    PageChooserPanel,
    TabbedInterface,
)
from wagtail.images.edit_handlers import ImageChooserPanel


class EventTag(TaggedItemBase):
    """
    Relation intermédiaire entre un évènement et une étiquette.
    """

    content_object = ParentalKey(
        'Event', on_delete=models.CASCADE, related_name='tagged_event_items'
    )


class EventQuerySet(models.QuerySet):
    def live(self):
        """Filtre avec les évènements publiés."""
        return self.filter(live=True)

    def upcoming(self):
        """Filtre avec les évènements à venir."""
        now = timezone.now().date()
        return self.filter(
            Q(date_start__gte=now, date_end__isnull=True)
            | Q(date_end__gte=now)
        )


class Event(ClusterableModel):
    """
    Un évènement à afficher dans l'agenda.
    """

    # Description
    title = models.CharField("titre", max_length=128)
    tags = ClusterTaggableManager(
        through=EventTag,
        blank=True,
        verbose_name="étiquettes",
        help_text=(
            "Une liste d'une ou plusieurs étiquettes séparées par une "
            "virgule."
        ),
    )
    image = models.ForeignKey(
        'wagtailimages.Image',
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name='+',
        verbose_name="illustration",
    )

    # Détails
    summary = models.TextField("résumé", blank=True)
    related_page = models.ForeignKey(
        'wagtailcore.Page',
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name='+',
        verbose_name="page liée",
        help_text=(
            "Une page ou un article du site en rapport avec cet évènement, "
            "vers lequel faire pointer pour en savoir plus."
        ),
    )
    external_link = models.URLField(
        "lien externe",
        blank=True,
        help_text=(
            "L'URL d'un site externe en rapport avec cet évènement, vers "
            "lequel faire pointer pour en savoir plus."
        ),
    )

    # Date et heure
    date_start = models.DateField("date de début")
    date_end = models.DateField(
        "date de fin",
        null=True,
        blank=True,
        help_text=(
            "La date de fin de l'évènement s'il est sur plusieurs jours."
        ),
    )
    time_start = models.TimeField("heure de début", null=True, blank=True)
    time_end = models.TimeField("heure de fin", null=True, blank=True)

    place = models.ForeignKey(
        'Place',
        blank=True,
        null=True,
        on_delete=models.CASCADE,
        verbose_name="lieu",
    )

    # Promotion
    live = models.BooleanField(
        "publié",
        default=True,
        help_text="Si l'évènement doit apparaître sur le site et l'agenda.",
    )
    slug = models.SlugField(
        "slug",
        allow_unicode=True,
        max_length=255,
        blank=True,
        unique=True,
        help_text=mark_safe(
            "Le nom de la page comme elle apparaîtra dans l'URL (e.g "
            "<code>http://domain.com/events/[my-slug]/</code>). Laissez-le "
            "vide pour en générer un automatiquement."
        ),
    )

    objects = EventQuerySet.as_manager()

    # Attributs Wagtails
    # ------------------

    # Panneaux pour l'administration
    content_panels = [
        MultiFieldPanel(
            [
                FieldPanel('title'),
                FieldPanel('tags'),
                ImageChooserPanel('image'),
            ],
            heading="Description",
        ),
        MultiFieldPanel(
            [
                FieldPanel(
                    'summary', widget=forms.Textarea(attrs={'rows': 2})
                ),
                PageChooserPanel('related_page'),
                FieldPanel('external_link'),
            ],
            heading="Détails",
        ),
        MultiFieldPanel(
            [
                FieldRowPanel(
                    [FieldPanel('date_start'), FieldPanel('date_end')]
                ),
                FieldRowPanel(
                    [FieldPanel('time_start'), FieldPanel('time_end')]
                ),
            ],
            heading="Date et heure",
        ),
        FieldPanel('place'),
    ]
    promote_panels = [
        MultiFieldPanel(
            [
                FieldPanel(
                    'slug',
                    widget=forms.TextInput(
                        attrs={
                            # En laissant l'id par défaut (id_slug), le slug va
                            # être automatiquement généré avec le titre en
                            # JavaScript par Wagtail, ce qu'on ne veut pas ici.
                            'id': 'event_slug'
                        }
                    ),
                ),
                FieldPanel('live'),
            ],
            heading="Configuration générale",
        )
    ]

    edit_handler = TabbedInterface(
        [
            ObjectList(content_panels, heading="Contenu"),
            ObjectList(promote_panels, heading="Promotion"),
        ]
    )

    class Meta:
        verbose_name = 'évènement'
        verbose_name_plural = 'évènements'
        ordering = ('date_start', 'date_end', 'time_start')

    def __str__(self):
        return self.title

    @cached_property
    def is_one_day(self):
        """Détermine si l'évènement est sur une seule journée."""
        return not self.date_end or self.date_start == self.date_end

    @cached_property
    def str_date(self):
        """Retourne la date formatée de l'évènement."""
        if not self.date_start:
            return ''
        date_start = date_format(self.date_start, 'd b. Y')
        if self.is_one_day:
            return date_start
        date_end = date_format(self.date_end, 'd b. Y')
        return "{} - {}".format(date_start, date_end)

    str_date.short_description = "Date"
    str_date.admin_order_field = 'date_start'

    @cached_property
    def str_time(self):
        """Retourne l'heure formatée de l'évènement."""
        if not self.time_start:
            return "Toute la journée"
        time_start = time_format(self.time_start, 'H:i')
        if not self.time_end:
            return time_start
        time_end = time_format(self.time_end, 'H:i')
        return "{} - {}".format(time_start, time_end)

    str_time.short_description = "Heure"
    str_time.admin_order_field = 'time_start'

    def full_clean(self, *args, **kwargs):
        """Prépare certains champs avant la validation."""
        if not self.slug:
            # Tente de générer un slug unique, une erreur sera ajoutée
            # lors du `clean()` si le slug est encore vide
            self.slug = self.generate_unique_slug()

        # Supprime la date de fin si c'est le même jour
        if self.date_end and self.date_end == self.date_start:
            self.date_end = None

        # Supprime l'heure de fin s'il n'y a pas de début
        if self.time_end and not self.time_start:
            self.time_end = None

        super().full_clean(*args, **kwargs)

    def clean(self):
        """Complète la validation des champs."""
        errors = {}

        # Vérifie la date et l'heure de fin
        if self.date_end and self.date_end < self.date_start:
            errors[
                'date_end'
            ] = "La date de fin doit être supérieure à celle de début."
        if self.time_end and self.time_end < self.time_start:
            errors[
                'time_end'
            ] = "L'heure de fin doit être supérieure à celle de début."

        if not self.slug:
            errors['slug'] = "Le slug n'a pas pu être généré automatiquement."

        if errors:
            raise ValidationError(errors)

    def get_absolute_url(self):
        """Retourne l'URL vers les détails de l'évènement."""
        return reverse('events:detail', args=[self.slug])

    def generate_unique_slug(self):
        """Génère un slug unique à partir des champs de l'évènement."""
        if not self.date_start or not self.title:
            return ''

        base_slug = slugify(
            '{}-{}'.format(self.date_start.strftime('%Y%m%d'), self.title)
        )

        suffix = 1
        candidate_slug = base_slug
        events = Event.objects.all()
        if self.pk is not None:
            events = events.exclude(pk=self.pk)

        while events.filter(slug=candidate_slug).exists():
            candidate_slug = '{}-{}'.format(base_slug, suffix)
            suffix += 1

        return candidate_slug


class Place(models.Model):
    """
    Un lieu d'un évènement.
    """

    name = models.CharField("nom", max_length=128)
    street = models.TextField("rue", max_length=255, blank=True)
    zip_code = models.CharField("code postal", max_length=16, blank=True)
    city = models.fields.CharField("ville", max_length=128, blank=True)
    image = models.ForeignKey(
        'wagtailimages.Image',
        null=True,
        blank=True,
        related_name='+',
        on_delete=models.SET_NULL,
        verbose_name="image du lieu",
    )

    # Attributs Wagtails
    # ------------------

    # Panneaux pour l'administration
    panels = [
        MultiFieldPanel(
            [FieldPanel('name'), ImageChooserPanel('image')],
            heading="Description",
        ),
        MultiFieldPanel(
            [
                FieldPanel('street'),
                FieldRowPanel([FieldPanel('zip_code'), FieldPanel('city')]),
            ],
            heading="Adresse",
        ),
    ]

    class Meta:
        verbose_name = "lieu"
        verbose_name_plural = "lieux"

    def __str__(self):
        return self.name
