import datetime

from django.core.exceptions import ValidationError
from django.utils.formats import date_format, time_format

import pytest

from ..models import Event


@pytest.mark.django_db
class TestEvent:
    def test_str(self, oneday_event):
        assert str(oneday_event) == "Event test - One day"

    def test_event_is_one_day(self, oneday_event, manydays_event):
        assert oneday_event.is_one_day
        assert not manydays_event.is_one_day

    def test_str_date(self, oneday_event, manydays_event):
        jp1 = date_format(
            datetime.date.today() + datetime.timedelta(days=1),
            'd b. Y',
        )
        jp4 = date_format(
            datetime.date.today() + datetime.timedelta(days=4),
            'd b. Y',
        )
        assert oneday_event.str_date == jp1
        assert manydays_event.str_date == "{} - {}".format(jp1, jp4)

    def test_str_time(self, oneday_event, manydays_event):
        assert oneday_event.str_time == 'Toute la journée'
        start = datetime.time(hour=8)
        other_event = Event(
            title="test",
            date_start=datetime.date.today(),
            time_start=start,
        )
        assert other_event.str_time == time_format(start, 'H:i')
        end = datetime.time(hour=18, minute=30)
        res = "{} - {}".format(
            time_format(start, 'H:i'), time_format(end, 'H:i')
        )
        assert manydays_event.str_time == res

    def test_full_clean(self):
        event = Event(
            title="test",
            date_start=datetime.date.today(),
            date_end=datetime.date.today(),
            time_end=datetime.time(hour=18, minute=30),
        )
        event.full_clean()
        assert event.slug
        assert not event.date_end
        assert not event.time_end

    def test_clean(self):
        with pytest.raises(ValidationError) as error:
            event = Event(
                title="test",
                date_start=datetime.date.today(),
                date_end=datetime.date.today() - datetime.timedelta(days=1),
                time_start=datetime.time(hour=19, minute=30),
                time_end=datetime.time(hour=18, minute=30),
            )
            event.clean()
        expected = {
            'date_end': [
                'La date de fin doit être supérieure à celle de début.'
            ],
            'time_end': [
                "L'heure de fin doit être supérieure à celle de début."
            ],
            'slug': ["Le slug n'a pas pu être généré automatiquement."],
        }
        assert dict(error.value) == expected
