from collections import namedtuple

from wagtail.core import blocks
from wagtail.core.blocks.field_block import BooleanBlock

"""
Le style d'un élément, définis par :
* value : l'identifiant du style
* label : le libellé à afficher
* css_class : la ou les classes CSS à appliquer
"""
Style = namedtuple('Style', ['value', 'label', 'css_class'])


class StylesChoiceBlock(blocks.ChoiceBlock):
    """
    Bloc de base pour le choix du style d'un élément.

    La liste des choix possibles est construite automatiquement. Elle utilise
    pour cela `styles`, qui doit être une liste d'objet `Style`.
    """

    def __init__(self, styles, default=None, **kwargs):
        if 'choices' in kwargs:
            raise ValueError("Unexpected 'choices' argument")
        choices = tuple((s.value, s.label) for s in styles)

        if default is None:
            default = choices[0][0]

        self.styles = {s.value: s for s in styles}
        super().__init__(choices, default, **kwargs)

    def get_css_class(self, value):
        """Retourne la classe CSS pour un style."""
        style = self.styles.get(value, None)
        return style.css_class if style else ''


class StylizedBlockMixin:
    """
    Étends la valeur de `blocks.StructValue` d'un bloc `blocks.StructBlock`,
    afin d'ajouter la classe CSS de son style dans un attribut css_class.

    Lors du rendu du bloc, la classe CSS sera donc accessible ainsi :

        {{ value.css_class }}
    """

    """Le nom du bloc définissant le style, de type `StylesChoiceBlock`."""
    style_block_name = 'style'

    def get_style_block(self):
        """Retourne le bloc de type `StylesChoiceBlock`."""
        return self.child_blocks.get(self.style_block_name, None)

    def _to_struct_value(self, block_items):
        """Ajoute la classe CSS du style dans l'attribut css_class."""
        value = super()._to_struct_value(block_items)

        style_block = self.get_style_block()
        style_value = value.get(self.style_block_name, None)

        if style_block and style_value is not None:
            value.css_class = style_block.get_css_class(style_value)
        else:
            value.css_class = None

        return value


class HideableStructBlock(blocks.StructBlock):
    """
    Permet d'ajouter un attribut hidden à un bloc pour lui donner
    la possibilité de ne pas être affiché au public
    """

    hidden = BooleanBlock(
        required=False,
        label="Bloc caché",
        help_text=(
            "Cocher ici si le bloc ne doit pas être affiché au public. "
            "Tous les blocs contenus dans celui-ci seront cachés également."
        ),
        default=False,
    )

    def render(self, value, context=None):
        if not value['hidden']:
            return super().render(value, context=context)
        return ""
